import { mve } from 'mve-core/util';
export interface TargetNumberChange {
    from?: number;
    value: mve.Value<number>;
    to: number;
}
export interface TargetTChange<T> {
    from?: T;
    value: mve.Value<T>;
    to: T;
    valueToNumber(t: T): number;
    numberToValue(n: number): T;
}
export declare function targetTChange<T>(to: T, p: Omit<TargetTChange<T>, 'to'>): TargetTChange<T>;
export declare type TargetChange = TargetNumberChange | TargetTChange<any>;
/**业务值渐变动画*/
export declare function targetAnimationOf(p: {
    data: TargetChange | TargetChange[];
    begin?(): void;
    end?(): void;
    change?: AnimationFun;
    duration?: number;
}): void;
/**
 * 产生动画
 * @param xp
 */
export declare function animationOf(xp: {
    /**持续时间 毫秒，1000毫秒是1秒*/
    duration: number;
    /**开始的值 */
    min?: number;
    /**结束的值 */
    max: number;
    /**回调位移,与时间 */
    call?(num: number, t: number): void;
    /**位移变化，与时间变化 */
    diff?(num: number, t: number): void;
    /**使用的动画 */
    change: AnimationFun;
    /**结束时调用 */
    end?(): void;
}): () => void;
interface AnimationPkg {
    easeIn: AnimationFun;
    easeOut: AnimationFun;
    easeInOut: AnimationFun;
}
export interface AnimationFun {
    /**
     * @param t current time（当前时间）
     * @param b beginning value（初始值）
     * @param c change in value（变化量）
     * @param d duration（持续时间）
     * @returns 位移
     */
    (t: number, b: number, c: number, d: number, e?: number, f?: number): number;
}
export declare const Animation: {
    Linear: AnimationFun;
    Quad: AnimationPkg;
    Cubic: AnimationPkg;
    Quart: AnimationPkg;
    Quint: AnimationPkg;
    Sine: AnimationPkg;
    Expo: AnimationPkg;
    Circ: AnimationPkg;
    Elastic: AnimationPkg;
    Back: AnimationPkg;
    Bounce: AnimationPkg;
};
export interface DrawOfBezier3 {
    start: number;
    c1: number;
    end: number;
    c2: number;
}
/***
 * 3阶贝塞尔，X方向与Y方向的计算是一样的。
 * @param startY 开始点Y
 * @param c1Y 开始点的控制点1
 * @param endY 结束点Y
 * @param c2Y 结束点的控制点2
 * @returns 动画函数
 */
export declare function drawOfBezier3(p: DrawOfBezier3): AnimationFun;
export {};
