import { EOChild } from 'mve-core/childrenBuilder';
import { RenderModelChildren } from 'mve-core/modelChildren';
import { BuildResult, mve } from 'mve-core/util';
import { DOMNode, EventMap, InitFun } from './index';
/**
 * 简单的tsx支撑
 * 如在tsconfig中配置
        "jsx": "react",
        "jsxFactory":"createElement",
 * 需要的地方导入
        import { createElement,Dom,Fragment,JSX, Svg, SvgFragment } from 'mve-dom/tsxSupport'
 * 可能需要全局导入
        declare namespace JSX {
            type IntrinsicElements=import("mve-dom/tsxSupport").JSX.IntrinsicElements
            type Element=import("mve-dom/tsxSupport").JSX.Element
            type ElementChildrenAttribute=import('mve-dom/tsxSupport').JSX.ElementChildrenAttribute
        }
 */
export declare function createElement<A extends (...vs: any[]) => any>(type: A, ...vs: Parameters<A>): ReturnType<A>;
export declare type TsxChild = string | number | boolean | EOChild<Node>;
export interface DOMTsxProp extends Omit<DOMNode, 'children'> {
    children?: TsxChild | TsxChild[];
}
export declare function Dom(props: DOMTsxProp, ...children: TsxChild[]): EOChild<Node>;
export declare function fromTsxChild(child: TsxChild): EOChild<Node>;
export declare function fromTsxChildren(children: TsxChild[]): EOChild<Node>[];
export declare function mergeFromTsxChildren(childrenA: TsxChild | TsxChild[] | void, childrenB: TsxChild[]): EOChild<Node>[];
export declare function Fragment(props?: {
    init?: BuildResult;
    children?: TsxChild | TsxChild[];
}, ...children: TsxChild[]): EOChild<Node>;
export interface SvgTsxProp extends Omit<DOMNode, 'children'> {
    type: string;
    children?: EOChild<Node> | EOChild<Node>[];
}
export declare function Svg(props: SvgTsxProp, ...children: EOChild<Node>[]): EOChild<Node>;
export declare function mergeFromEOChild(childrenA: EOChild<Node> | EOChild<Node>[] | void, childrenB: EOChild<Node>[]): EOChild<Node>[];
export declare function SvgFragment(props?: {
    init?: BuildResult;
    children?: EOChild<Node> | EOChild<Node>[];
}, ...children: EOChild<Node>[]): EOChild<Node>;
declare type TextType = string | number | boolean;
export declare type TextChild = (() => TextType) | TextType;
export declare function Text(props?: {
    init?: InitFun | InitFun[];
    event?: EventMap;
    children?: TextChild | TextChild[];
}, ...children: TextChild[]): EOChild<Node>;
export declare function mergeFromTextChild(childrenA: TextChild | TextChild[] | undefined, childrenB: TextChild[]): TextChild;
export declare function strToFun<T>(child: T | (() => T)): () => T;
export declare function DelayMap<T>(props: {
    array(): T[];
    key(v: T): any;
    children: RenderModelChildren<T, EOChild<Node>>;
}, children: RenderModelChildren<T, EOChild<Node>>): EOChild<Node>;
export declare function DelayIf<T>(props: {
    array(): T;
    key(v: T): any;
    children: (me: mve.LifeModel, row: T) => EOChild<Node>;
}, children: (me: mve.LifeModel, row: T) => EOChild<Node>): EOChild<Node>;
export declare namespace JSX {
    /**禁止所有内建小写元素 */
    interface IntrinsicElements {
    }
    /**与mve-core核心类型的兼容性 */
    type Element = EOChild<Node>;
    /**
     * TSX内容元素类型，需要先声明
     */
    interface ElementChildrenAttribute {
        children: any;
    }
}
export {};
